/*
    File:            consol_awf1c.cpp
    Author:          Tomas Krejci, 12/09/2008, revised 23/08/2012
    Purpose:         material model for saturated-nonsaturated one-phase flow in deforming medium
    sources:         Lewis and Schrefler's book pp. 86-89, initial material parameters are set for benchmark on page n. 168;
    another sources: geomkp.pdf from michal Sejnoha;
                     
                     W. R. Gardner, Some steady-state solutions of the unsaturated moisture flow equation 
                     to evaporation from a water table, Soil Science 85 (1958), no. 4, 228-232;
                     
                     D. M. Potts and L. Zdravkovic, Finite element analysis in geotechnical engineering theory, 
		     Thomas Telford, London, 1999;

		     M. Th. van Genuchten, A closed equation for predicting the hydraulic conductivity of
		     unsaturated soils, Journal Soil Science Society of America 44 (1980), 892-898;
		     
		     P. Kuklik, J. Mares and M. Sejnoha (1999) The structural strenght of soil
		     from the isotropic consolidation point of view. Computational Mechanics for the Next
		     Millenium. Wang, Lee, Ang (ed.), Vol. 2, Elsevier, Singpore, 797-802;

		     P. Kuklik (2010) Preconsolidation, Structural Strength of soil, and its effect on subsoil 
		     upper structure intraction; Professor thesis

    unknowns:        number of unknowns=1, pw = liqud water pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "constrel.h"
#include "onemediumc.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "consol_awf1c.h"
#include "intpoints.h"


con_awf1matc::con_awf1matc()
{
  alpha = 1.0; //Biot's constant
  //
  phi0 = 0.297;//initial porosity [-]
  //
  rhos = 2000.0;//solid grain density [kg/m^3]
  //
  rhow = 1000.0;//water density [kg/m^3]
}

con_awf1matc::~con_awf1matc()
{}


/**
   function reads parameters
   
   @param in - input file

   12/9/2008, TKr
*/
void con_awf1matc::read(XFILE *in)
{
  
  xfscanf (in,"%k%m","waterflowtype",&waterflowtype_kwdset, &model_type);
  
  switch (model_type){
  case lewis_and_schrefler_coup:{//Lewis and Schrefler's book
    xfscanf (in,"%lf  %lf  %lf", &alpha, &phi0, &rhos);
    break;
  }
  case kuklik_camclay_coup:{//fully saturated isotropic consolidation according to Kuklik
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
}


/**
   function prints parameters
   
   @param out - output file

   12/9/2008, TKr
*/
void con_awf1matc::print(FILE *out)
{
  fprintf(out,"  %d",model_type);
  switch (model_type){
  case lewis_and_schrefler_coup:{//Lewis and Schrefler's book
    fprintf (out,"%lf  %lf  %lf", alpha, phi0, rhos);
    break;
  }
  case kuklik_camclay_coup:{//fully saturated isotropic consolidation according to Kuklik
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 


}



/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  double g;
  
  pw = Cmu->ip[ipp].av[0];
  
  f = get_fu1(pw);
  g = Tp->gr1;//vector of gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g;
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  double *g;
  g = new double [2];
    
  pw = Cmu->ip[ipp].av[0];
  
  f = get_fu1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  double *g;  
  g = new double [3];
  
  pw = Cmu->ip[ipp].av[0];

  f = get_fu1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  g[2] = Tp->gr3;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
  d[2][0] = f*g[2];
}



/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure

   @retval sw - degree of saturation

   12/9/2008, TKr
*/
double con_awf1matc::get_sw(double pw)
{
  double sw,kr;
  double hp,theta,expm;
  
  switch (model_type){
  case lewis_and_schrefler_coup:{//Lewis and Schrefler's book
    sw = 1.0 - (1.9722e-11)*pow(-pw,2.4279);//p. 167
    break;
  }
  case kuklik_camclay_coup:{//fully saturated isotropic consolidation according to Kuklik
    sw = 1.0;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(sw);
}


/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double con_awf1matc::get_phi()
{
  double phi;

  phi = phi0;

  return(phi);
}



/**
   function returns volume density of soil skeleton

   @retval rhos - volume density of concrete skeleton

   12/9/2008, TKr
*/
double con_awf1matc::get_rhos()
{
  return(rhos);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kuw - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_kuw(double pw)
{
  double s,kuw;
  
  switch (model_type){
  case lewis_and_schrefler_coup:{//Lewis and Schrefler's book  
    
    s = get_sw(pw);
    
    kuw = -alpha*s;//p. 88
    break;
  }
  case kuklik_camclay_coup:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    kuw = 1.0;//tady zkontrolovat odvozeni??!!
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(kuw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kwu - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_kwu(double pw)
{
  double kwu;

  kwu = 0.0;

  return(kwu);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capuw - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_capuw(double pw)
{
  double capuw;
  
  capuw = 0.0;

  return(capuw);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capwu - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_capwu(double pw)
{
  double s,capwu;
  
  switch (model_type){
  case lewis_and_schrefler_coup:{//Lewis and Schrefler's book  
    s = get_sw(pw); 
    capwu = alpha*s;//p. 88
    break;
  }
  case kuklik_camclay_coup:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    capwu = -1.0;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(capwu);

}


/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fu1 - first part for right-hand side for balance equation

   12/9/2008, TKr
*/
double con_awf1matc::get_fu1(double pw)
{
  double sw,n,fu1;  
  
  sw = get_sw(pw);
  n = phi0;
  
  //fu1 = rhos*(1-n) + sw*n*rhow;//p. 88
  fu1 = 0.0;//debug = no own weigth

  return(fu1);

}
